@@ -53,7 +53,7 @@ def eval_pickled(f_s, args_s):
     try:
         get_logger(f).info("start {}{} in process {}".format(f.__name__, args, os.getpid()))
         result = f(*args)
-        get_logger(f).info("got result {} from process {}".format(result, os.getpid()))
+        get_logger(f).info("got result {!r} from process {}".format(result, os.getpid()))
         return result
     except Exception, e:
         get_logger(f).error("got exception: {}".format(e))
