@@ -178,7 +178,7 @@ class _CachedFunction(object):
         try:
             origin_str = serialize_tuple(origin).replace("/", "%")
         except Exception, e:
-            raise Exception("can not pickle %r beause of:\n%s" % (e.message,))
+            raise Exception("can not pickle %r beause of: %s" % (result, e.message,))
 
         if len(origin_str) > 100:
             origin_str = "[TOO_LONG]"
@@ -214,7 +214,8 @@ class _CachedFunction(object):
     def _get_origin(self, arg):
         get_origin = self._lookup_get_origin_function(arg)
         if get_origin is not None:
-            return get_origin(arg)
+            origin = get_origin(arg)
+            return origin if origin is not None else "XXX"
         if isinstance(arg, (tuple, list)):
             return tuple(self._get_origin(item) for item in arg)
         elif isinstance(arg, basestring):
@@ -234,14 +235,14 @@ class _CachedFunction(object):
             return set_origin(obj, origin)
         return obj
 
-    def __call__(self, *args):
-        hash_code = self._compute_hash(args)
+    def __call__(self, *args, **kw):
+        hash_code = self._compute_hash(args + tuple(sorted(kw.items())))
         if self._contains(hash_code):
             with self._lock:
                 self._hit_counter.value += 1
             self._logger.info("cache hit for %s" % hash_code)
             return self._get(hash_code)
-        result = self.function(*args)
+        result = self.function(*args, **kw)
         origin = self._get_origin(args)
         result = self._set_origin(result, origin)
         self._logger.info("new result for %s" % hash_code)
